/*

 This library is copyrighted(C) 1999 by WeeSan Lee <weesan@employees.org>, 
 with all rights reserved.  

 This program is free software;  please feel free to redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 (version 2) as published by the Free Software Foundation.  It is 
 distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 */

// $Id: debug.h,v 1.1 2005/05/26 22:36:10 weesan Exp $

#ifndef DEBUG_H
#define DEBUG_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define DEBUG_INIT()          static Debug cDebug
#define DEBUG_REGISTER(name)  Debug name::cDebug(#name)
#define DEBUG_ENABLE()        cDebug.enable()
#define DEBUG(format...)      cDebug.debug(format)

#define DEBUG_TOTAL          128

class DebugRegistry {
private:
    char *pzcName;
    bool bEnable;
public:
    DebugRegistry(const char *pzcName) :
	pzcName(strdup(pzcName)),
	bEnable(false) {
    }
    ~DebugRegistry(void) {
	free(pzcName);
    }
    const char *name(void) const {
	return (pzcName);
    }
    bool enable(void) const {
	return (bEnable);
    }
    void enable(bool bOn) {
	bEnable = bOn;
    }
};

class Debug {
private:
    static DebugRegistry *pcDebugRegistry[DEBUG_TOTAL];
    static int iTotal;
    int iRegistry;

public:
    Debug(void) : iRegistry(-1) {
    }
    Debug(const char *pzcClassName) : iRegistry(-1) {
	if (iTotal >= DEBUG_TOTAL) {
	    fprintf(stderr,
		    "Ran out of Debug registry, "
		    "try to increase DEBUG_TOTAL\n");
	} else {
	    DebugRegistry *pcNewRegistry = new DebugRegistry(pzcClassName);
	    if (!pcNewRegistry) {
		fprintf(stderr, "Not enough memory to allocate debug for '%s'",
			pzcClassName);
	    } else {
		pcDebugRegistry[iTotal] = pcNewRegistry;
		iRegistry = iTotal++;
	    }
	}
    }
    static void on(const char *pzcClassName) {
	for (int i = 0; i < DEBUG_TOTAL; i++) {
	    if (pcDebugRegistry[i] &&
		strcmp(pcDebugRegistry[i]->name(), pzcClassName) == 0) {
		pcDebugRegistry[i]->enable(true);
		break;
	    }
	}
    }
    static void off(const char *pzcClassName) {
	for (int i = 0; i < DEBUG_TOTAL; i++) {
	    if (pcDebugRegistry[i] &&
		strcmp(pcDebugRegistry[i]->name(), pzcClassName) == 0) {
		pcDebugRegistry[i]->enable(false);
		break;
	    }
	}
    }
    bool enable(void) const {
	return (iRegistry >= 0 && iRegistry < DEBUG_TOTAL &&
		pcDebugRegistry[iRegistry]->enable());
    }
    void debug(const char *pzcFormat, ...) {
	if (enable()) {
	    va_list ap;
	    char pzcPrettyFormat[256];

	    va_start(ap, pzcFormat);
	    snprintf(pzcPrettyFormat, sizeof(pzcPrettyFormat), "%s: %s\n",
		     pcDebugRegistry[iRegistry]->name(), pzcFormat);
	    vfprintf(stderr, pzcPrettyFormat, ap);
	    va_end(ap);
	}
    }
};

#endif // DEBUG_H
