// $Id: link.cc,v 1.4 2005/06/13 06:06:53 weesan Exp $

#include "link.h"
#include "simulation.h"

DEBUG_REGISTER(Link);

// Global variable
Links cLinks;

///////////////////////////////////

// Static variable
link_id_t Link::tNextId = 0;

///////////////////////////////////

static void link2node(void *pvNode, void *pvLink, void *pvPacket) {
    Node *pcNode = (Node *)pvNode;
    Link *pcLink = (Link *)pvLink;
    Packet *pcPacket = (Packet *)pvPacket;

    pcNode->rx(pcPacket, pcLink);
}

///////////////////////////////////

// Constructor.  Assume single directional.
Link::Link(Node *pcSrcNode, Node *pcDstNode,
	   unsigned int uiBandwidth, int iDelay) :
    Object("Link"),
    tId(nextId()),
    pcSrcNode(pcSrcNode),
    pcDstNode(pcDstNode),
    uiBandwidth(uiBandwidth),
    iDelay(iDelay) {
    cLinks.add(this);
    pcSrcNode->addLink(this);
    DEBUG("%d (%d -> %d) created", srcNode()->id(), dstNode()->id(), id());
}

Link::~Link(void) {
    DEBUG("%d destroyed", id());
}

bool Link::rx(Packet *pcPacket, Node *pcNode) {
    DEBUG("%d rx packet %d from node %d", id(), pcPacket->id(), pcNode->id());
    if (pcNode == srcNode()) {
	/*
	 * XXX: some optimization here.  This is mainly for wireless.
	 * When the delay is 0, just have dst node to receive the packet
	 * directly instead of going thru scheduler which would takes ages
	 * to finish.
	 */
	if (delay()) {
	    AT(delay(), link2node, dstNode(), this, pcPacket);
	} else {
	    link2node(dstNode(), this, pcPacket);
	}
/*
    } else if (pcNode == dstNode()) {
        AT(delay(), link2node, srcNode(), this, pcPacket);
	return (true);
*/
    } else {
	DEBUG("%d no connection to node %d, drop packet %d",
	      id(), pcNode->id(), pcPacket->id());
	pcPacket->destroy();
	return (false);
    }
}
