// $Id: link.h,v 1.2 2005/06/04 09:45:42 weesan Exp $

#ifndef LINK_H
#define LINK_H

#include <map>
#include "object.h"
#include "node.h"
#include "packet.h"
#include "debug.h"

#define LINK_T3    44740000
#define LINK_T1    1544000
#define LINK_1M    1024000
#define LINK_512K  512000
#define LINK_256K  256000
#define LINK_128K  128000
#define LINK_56K   56000
#define LINK_19K   19200

typedef unsigned int link_id_t;

using namespace std;

class Link : public Object {
    DEBUG_INIT();

private:
    static link_id_t tNextId;

private:
    link_id_t tId;
    Node *pcSrcNode, *pcDstNode;   // End nodes of the link
    unsigned int uiBandwidth;      // Bandwidth of the link in bps
    int iDelay;                    // Propagation delay in ms
                                   // Assume 0 for wireless for now

public:
    Link(Node *pcSrcNode, Node *pcDstNode,
	 unsigned int uiBandwidth, int iDelay = 0);
    virtual ~Link(void);
    static link_id_t nextId(void) {
	return (tNextId++);
    }
    link_id_t id(void) const {
	return (tId);
    }
    virtual void print(ostream &os) {
	os << name() << id();
    }
    Node *srcNode(void) const {
	return (pcSrcNode);
    }
    Node *dstNode(void) const {
	return (pcDstNode);
    }
    unsigned int bandwidth(void) const {
	return (uiBandwidth);
    }
    int delay(void) const {
	return (iDelay);
    }
    /*
     * We consider 2 directional links, which has the same end nodes
     * regardless of their direction, as the same
     */
    bool operator==(Link &l) {
	return (pcSrcNode == l.srcNode() || pcDstNode == l.dstNode());
    }
    virtual bool rx(Packet *pcPacket, Node *pcNode);
};

class Links : public multimap<Node *, Link *> {
public:
    ~Links(void) {
	for (iterator itr = begin(); itr != end(); itr++) {
	    delete (itr->second);
	}
    }
    void add(Link *pcLink) {
	insert(multimap<Node *, Link *>::value_type(pcLink->srcNode(),
						    pcLink));
    }
    Link *find(Node *pcSrcNode, Node *pcDstNode) {
	for (iterator itr = multimap<Node *, Link *>::find(pcSrcNode);
	     itr != end() && itr->first == pcSrcNode;
	     itr++) {
	    if (itr->second->dstNode() == pcDstNode) {
		return (itr->second);
	    }
	}
	return (NULL);
    }
};

// Global variable
extern Links cLinks;

#endif // LINK_H
