// $Id: node.h,v 1.3 2005/06/11 22:47:09 weesan Exp $

#ifndef NODE_H
#define NODE_H

#include <vector>
#include <map>
#include "object.h"
#include "queue.h"
#include "debug.h"

typedef unsigned int node_id_t;

using namespace std;

// Forward declaration
class Link;
class Packet;
class RoutingTable;

class Node : public Object, public map<Link *, Link *> {
    DEBUG_INIT();

private:
    static node_id_t tNextId;      // Keep track of next available node id

private:
    node_id_t tId;                 // Unique node id
    Queue *pcQueue;                // Queue
    RoutingTable *pcRoutingTable;    // Routing table

private:
    // Check to see if the link is a valid link to the node
    bool validLink(Link *pcLink) {
	return (find(pcLink) != end());
    }
    // Transmit a packet to a link directly
    void tx(Packet *pcPacket, Link *pcOutgoing);

public:
    Node(Queue *_pcQueue = NULL);
    virtual ~Node(void);
    static node_id_t nextId(void) {
	return (tNextId++);
    }
    node_id_t id(void) const {
	return (tId);
    }
    Queue *queue(void) const {
	return (pcQueue);
    }
    RoutingTable *routingTable(void) const {
	return (pcRoutingTable);
    }
    virtual void print(ostream &os) {
	os << name() << id();
    }
    void addLink(Link *pcLink) {
	insert(map<Link *, Link *>::value_type(pcLink, pcLink));
    }
    void addDefaultRoute(Link *pcDefault);
    bool operator==(const Node &rNode) {
	return (id() == rNode.id());
    }
    /*
     * Transmit a packet to a node.  Routing is required in this case to
     * figure out which link to send the packet to.
     * In the case when a packet is originate from the node, it's incoming
     * interface would be NULL
     */
    virtual void tx(Packet *pcPacket, Node *pcDstNode,
		    Link *pcIncoming = NULL);
    // Receive a packet from a link
    virtual void rx(Packet *pcPacket, Link *pcIncoming);
};

class Nodes : public vector<Node *> {
public:
    ~Nodes(void) {
	for (int i = 0; i < size(); i++) {
	    delete (*this)[i];
	}
    }
};

// Global variables
extern Nodes cNodes;

#endif // NODE_H
