// $Id: routing_table.cc,v 1.1 2005/06/04 09:45:42 weesan Exp $

#include "routing_table.h"
#include "simulation.h"

/*
 * RoutingTable::addEntry
 *
 * Add new entry into the routing table.
 * NOTE: New entry will override the old one.
 */
void RoutingTable::addEntry(node_id_t tDst, Link *pcOutgoing,
			    RouteEntry::Type eType) {
    // Remove the old one if any
    removeEntry(tDst);

    // Then add the new one
    insert(map<node_id_t, RouteEntry>::value_type(
	       tDst, RouteEntry(tDst, pcOutgoing, NOW(), eType)));
}

/*
 * RoutingTable::removeEntry
 *
 * Remove entry from the routing table
 */
void RoutingTable::removeEntry(node_id_t tDst) {
    iterator itr = find(tDst);
    if (itr != end()) {
	erase(itr);
    }
}

/*
 * RoutingTable::removeStaleEntries
 *
 * Remove all dynamic entries that are older than 'tNow'
 */
void RoutingTable::removeStaleEntries(time_t tNow) {
    for (iterator itr = begin(); itr != end(); ) {
	RouteEntry &e = itr->second;
	if (e.type() == RouteEntry::DYNAMIC &&
	    tNow - e.time() >= RT_ENTRY_TIMEOUT) {
	    iterator itr2 = itr;
	    itr++;
	    erase(itr2);
	} else {
	    itr++;
	}
    }
}

///////////////////////////////////

#ifdef DEBUG_ROUTING_TABLE

int main(void) {
    RoutingTable cRoutingTable;
    cRoutingTable.addEntry(0, new Node);
    printf("Routing Tale size = %d\n", cRoutingTable.size());
    cRoutingTable.addEntry(1, new Node);
    printf("Routing Tale size = %d\n", cRoutingTable.size());
    for (RoutingTable::iterator itr = cRoutingTable.begin();
	 itr != cRoutingTable.end();
	 itr++) {
	cout << itr->first << ' ' << *(itr->second.nextHop()) << endl;
    }
    cRoutingTable.addEntry(1, new Node);
    printf("Routing Tale size = %d\n", cRoutingTable.size());
    for (RoutingTable::iterator itr = cRoutingTable.begin();
	 itr != cRoutingTable.end();
	 itr++) {
	cout << itr->first << ' ' << *(itr->second.nextHop()) << endl;
    }
}

#endif
