// $Id: routing_table.h,v 1.1 2005/06/04 09:45:42 weesan Exp $

#ifndef ROUTING_TABLE_H
#define ROUTING_TABLE_H

#include <time.h>
#include <map>
#include "link.h"
#include "node.h"
#include "packet.h"

#define RT_DEFAULT_ROUTE  PK_BROADCAST_ADDR
#define RT_ENTRY_TIMEOUT  10000    // 10 secs

using namespace std;

class RouteEntry {
public:
    enum Type {
	STATIC,
	DYNAMIC
    };

private:
    node_id_t tDst;                // Destination node id
    Link *pcOutgoing;              // The interface that connects to next hop
    time_t tTime;                  // Time stamp of the entry in ms
    Type eType;                    // Either static or dynamic route entry

public:
    RouteEntry(node_id_t tDst, Link *pcOutgoing, time_t tTime, Type eType) :
	tDst(tDst),
	pcOutgoing(pcOutgoing),
	tTime(tTime),
	eType(eType) {
    }
    node_id_t dst(void) const {
	return (tDst);
    }
    Link *outgoing(void) const {
	return (pcOutgoing);
    }
    time_t time(void) const {
	return (tTime);
    }
    Type type(void) const {
	return (eType);
    }
};

class RoutingTable : public map<node_id_t, RouteEntry> {
public:
    RoutingTable(void) {
    }
    void addEntry(node_id_t tDst, Link *pcOutgoing,
		  RouteEntry::Type eType = RouteEntry::DYNAMIC);
    void removeEntry(node_id_t tDst);
    void removeStaleEntries(time_t tNow);
    Link *lookup(node_id_t tDst) {
	iterator itr = find(tDst);
	if (itr != end()) {
	    return (itr->second.outgoing());
	} else {
	    return (NULL);
	}
    }
};

#endif // ROUTING_TABLE_H
