// $Id: schedule.h,v 1.2 2005/06/02 03:45:17 weesan Exp $

#ifndef SCHEDULE_H
#define SCHEDULE_H

#include <time.h>
#include <iostream>
#include "object.h"

typedef void (*schedule_action_t)(void *, void *, void *);

using namespace std;

class Schedule {
private:
    time_t tTime;                  // Scheduled time in mini seconds
    schedule_action_t tAction;     // The action function
    Object *pcObject;              // Object in schedule
    void *pvParam1, *pvParam2;     // Associated 2 optional parameters

public:
    Schedule(time_t tTime, schedule_action_t tAction,
	     Object *pcObject, void *pvParam1, void *pvParam2) :
	tTime(tTime),
	tAction(tAction),
	pcObject(pcObject),
	pvParam1(pvParam1),
	pvParam2(pvParam2) {
    }
    time_t time(void) const {
	return (tTime);
    }
    Object *object(void) const {
	return (pcObject);
    }
    schedule_action_t action(void) const {
	return (tAction);
    }
    void *param1(void) const {
	return (pvParam1);
    }
    void *param2(void) const {
	return (pvParam2);
    }
    bool operator<(const Schedule &s) const {
	// Reverse the priority so that smaller time scheduled first
	return (tTime > s.time());
    }
    friend ostream &operator<<(ostream &os, const Schedule &s) {
	return (os << s.time() << ": " << *s.object());
    }
};

#endif // SCHEDULE_H
