// $Id: simulation.h,v 1.4 2005/06/12 09:31:10 weesan Exp $

#ifndef SIMULATION_H
#define SIMULATION_H

#include <stdlib.h>
#include <time.h>
#include <queue>
#include <iostream>
#include "object.h"
#include "schedule.h"
#include "debug.h"

// For convenience
#define SIM_1_MS     1
#define SIM_100_MS   100
#define SIM_1_SEC    1000

#define SIM_JILTER   5

#define AT           Simulation::at
/*
#define AT           \
    printf("%s:%d\n", __PRETTY_FUNCTION__, __LINE__); \
    Simulation::at
*/
#define NOW()        Simulation::now()
#define JILTER(j)    (random() % (j))

using namespace std;

class Simulation {
    DEBUG_INIT();

private:
    static priority_queue<Schedule> cSchedule;
    static time_t tTotalTime;      // Total simulation time in micro seconds
    static time_t tNow;            // Current time in the simulator

private:
    int iMaxX, iMaxY;              // Only used by wireless

public:
    Simulation(time_t tTotalTime) :
	iMaxX(0),
	iMaxY(0) {
	totalTime(tTotalTime);
    }
    Simulation(int iMaxX, int iMaxY, time_t tTotalTime) :
	iMaxX(iMaxX),
	iMaxY(iMaxY) {
	totalTime(tTotalTime);
    }
    ~Simulation(void) {
	DEBUG("Simulation ended at %d", NOW());
    }
    int maxX(void) const {
	return (iMaxX);
    }
    int maxY(void) const {
	return (iMaxY);
    }
    static time_t totalTime(void) {
	return (tTotalTime);
    }
    // The parameter 'tSec' is in seconds, convert it into mini-seconds
    static void totalTime(time_t tSec) {
	DEBUG("Simulation started for %d secs", tSec);
	tTotalTime = tSec * 1000;
    }
    static time_t now(void) {
	return (tNow);
    }
    static void now(time_t tTime) {
	tNow = tTime;
    }
    static time_t at(time_t tDeltaTime,
		     schedule_action_t tAction,
		     Object *pcObject,
		     void *pvParam1 = NULL,
		     void *pvParam2 = NULL) {
	time_t tTime = now() + tDeltaTime;
#ifdef SIM_JILTER
	tTime += random() % SIM_JILTER;
#endif
	if (tTime <= totalTime()) {
	    cSchedule.push(Schedule(tTime, tAction,
				    pcObject, pvParam1, pvParam2));
	    return (tTime);
	} else {
	    DEBUG("Scheduled time (%u) larger than simulation time (%u)",
		  tTime, totalTime());
	    return (0);
	}
    }
    static int scheduleSize(void) {
	return (cSchedule.size());
    }
    virtual void run(void);
};

extern Simulation cSimulation;

#endif // SIMULATION_H
