#include <math.h>
#include "wireless_link.h"
#include "wireless_node.h"
#include "dist_prob.h"

#define WIRELESS_LINK_NEIGHBOR_ERROR
//#define WIRELESS_LINK_DISTANCE_ERROR

DEBUG_REGISTER(WirelessLink);

#ifdef WIRELESS_LINK_DISTANCE_ERROR
/*
 * The error distribution below assumes that the further a node is trying
 * to transmit, the more likely the packet will be dropped due to errors
 */
static DistProb cDistanceError(
    9,
    5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 40.0, 55.0, 100.0);
#endif

/*
 * Wireless::rx()
 *
 * Implement a very simple error model here.
 * ie. the longer transmission is, it's more likely the packet would be dropped
 */
bool WirelessLink::rx(Packet *pcPacket, Node *pcNode) {
#ifdef WIRELESS_LINK_NEIGHBOR_ERROR
    static vector<DistProb *>cNeighborError(9);
    static bool bInit = false;

    if (!bInit) {
	bInit = true;
	for (int i = 0; i < cNeighborError.size(); i++) {
	    cNeighborError[i] = new DistProb(2, 50.0, (float)i);
	}
    }

    int iNeighbors = pcNode->size();
    if (iNeighbors >= cNeighborError.size()) {
	iNeighbors = cNeighborError.size() - 1;
    }

    if (cNeighborError[iNeighbors]->generate()) {
	DEBUG("%d packet %d is dropped due to link error",
	      id(), pcPacket->id());
	pcPacket->destroy();
	return (false);
    } else {
	return (Link::rx(pcPacket, pcNode));
    }
#elif defined(WIRELESS_LINK_DISTANCE_ERROR)
    WirelessNode *pcSrcNode = (WirelessNode *)srcNode();
    WirelessNode *pcDstNode = (WirelessNode *)dstNode();
    int x = pcSrcNode->x() - pcDstNode->x();
    int y = pcSrcNode->y() - pcDstNode->y();
    float fDistance = sqrt((float)(x * x + y * y));

    if (cDistanceError.data() <= (int)fDistance) {
	DEBUG("%d packet %d is dropped due to link error",
	      id(), pcPacket->id());
	pcPacket->destroy();
	return (false);
    } else {
	return (Link::rx(pcPacket, pcNode));
    }
#else
    return (Link::rx(pcPacket, pcNode));
#endif
}
