// $Id: wireless_node.h,v 1.6 2005/06/12 09:31:10 weesan Exp $

#ifndef WIRELESS_NODE_H
#define WIRELESS_NODE_H

#include <vector>
#include "node.h"
#include "link.h"
#include "packet.h"
#include "routing_table.h"
#include "debug.h"

using namespace std;

#define WL_RADIO_RANGE_DEFAULT      25  // in feet
//#define WL_BROADCAST_BACKOFF_TIMER  10  // in ms

class WirelessNode : public Node {
    DEBUG_INIT();

private:
    int iX, iY;                    // Coordinate
    int iR;                        // Communication range/radius
    unsigned int uiBandwidth;      // Bandwidth
    Packets *pcFwdedPkts;          // Keep track of forwarded packets

protected:
    void setupGradient(node_id_t tDst, Node *pcNeighborNode);

public:
    WirelessNode(int iX, int iY,
		 unsigned int uiBandwidth = LINK_19K,
		 int iR = WL_RADIO_RANGE_DEFAULT,
		 Queue *pcQueue = NULL,
		 Packets *pcFwdedPkts = NULL);
    virtual ~WirelessNode(void) {
	if (pcFwdedPkts) {
	    delete pcFwdedPkts;
	}
    }
    int x(void) const {
	return (iX);
    }
    int y(void) const {
	return (iY);
    }
    int r(void) const {
	return (iR);
    }
    unsigned int bandwidth(void) const {
	return (uiBandwidth);
    }
    Packets *fwdedPkts(void) {
	return (pcFwdedPkts);
    }
    bool inRange(WirelessNode *pcNode);
    /*
     * Broadcast a packet.
     * If 'pcIncoming' is NULL, it's a local originated packet
     */
    virtual void broadcast(Packet *pcPacket, Link *pcIncoming = NULL);
    /*
     * Receive a packet from a link.
     * In the case of broadcast, the link is NULL
     */
    virtual void rx(Packet *pcPacket, Link *pcIncoming);
};

#endif // WIRELESS_NODE_H
